@@ -35,6 +35,7 @@
 ****************************************************************************/
 
 #include "qquickapplicationwindow_p.h"
+#include "qquickcontentitem_p.h"
 #include "qquickoverlay_p.h"
 #include "qquickpopup_p_p.h"
 #include "qquickcontrol_p_p.h"
@@ -72,7 +73,7 @@ QT_BEGIN_NAMESPACE
     \image qtquickcontrols2-applicationwindow-wireframe.png
 
     \qml
-    import QtQuick.Controls 2.3
+    import QtQuick.Controls 2.12
 
     ApplicationWindow {
         visible: true
@@ -128,6 +129,8 @@ public:
         return window->d_func();
     }
 
+    QQmlListProperty<QObject> contentData();
+
     void relayout();
 
     void itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &diff) override;
@@ -262,7 +265,7 @@ void QQuickApplicationWindowPrivate::updateFont(const QFont &f)
 
 void QQuickApplicationWindowPrivate::resolveFont()
 {
-    QFont resolvedFont = font.resolve(QQuickControlPrivate::themeFont(QPlatformTheme::SystemFont));
+    QFont resolvedFont = font.resolve(QQuickTheme::font(QQuickTheme::System));
     setFont_helper(resolvedFont);
 }
 
@@ -284,7 +287,7 @@ void QQuickApplicationWindowPrivate::updatePalette(const QPalette &p)
 
 void QQuickApplicationWindowPrivate::resolvePalette()
 {
-    QPalette resolvedPalette = palette.resolve(QQuickControlPrivate::themePalette(QPlatformTheme::SystemPalette));
+    QPalette resolvedPalette = palette.resolve(QQuickTheme::palette(QQuickTheme::System));
     setPalette_helper(resolvedPalette);
 }
 
@@ -548,9 +551,10 @@ void QQuickApplicationWindow::setFooter(QQuickItem *footer)
 
     \sa contentItem
 */
-QQmlListProperty<QObject> QQuickApplicationWindow::contentData()
+QQmlListProperty<QObject> QQuickApplicationWindowPrivate::contentData()
 {
-    return QQmlListProperty<QObject>(contentItem(), this,
+    Q_Q(QQuickApplicationWindow);
+    return QQmlListProperty<QObject>(q->contentItem(), q,
                                      QQuickApplicationWindowPrivate::contentData_append,
                                      QQuickItemPrivate::data_count,
                                      QQuickItemPrivate::data_at,
@@ -572,7 +576,7 @@ QQuickItem *QQuickApplicationWindow::contentItem() const
 {
     QQuickApplicationWindowPrivate *d = const_cast<QQuickApplicationWindowPrivate *>(d_func());
     if (!d->contentItem) {
-        d->contentItem = new QQuickItem(QQuickWindow::contentItem());
+        d->contentItem = new QQuickContentItem(this, QQuickWindow::contentItem());
         d->contentItem->setFlag(QQuickItem::ItemIsFocusScope);
         d->contentItem->setFocus(true);
         d->relayout();
@@ -679,7 +683,7 @@ void QQuickApplicationWindow::setFont(const QFont &font)
     if (d->font.resolve() == font.resolve() && d->font == font)
         return;
 
-    QFont resolvedFont = font.resolve(QQuickControlPrivate::themeFont(QPlatformTheme::SystemFont));
+    QFont resolvedFont = font.resolve(QQuickTheme::font(QQuickTheme::System));
     d->setFont_helper(resolvedFont);
 }
 
@@ -758,7 +762,7 @@ void QQuickApplicationWindow::setPalette(const QPalette &palette)
     if (d->palette.resolve() == palette.resolve() && d->palette == palette)
         return;
 
-    QPalette resolvedPalette = palette.resolve(QQuickControlPrivate::themePalette(QPlatformTheme::SystemPalette));
+    QPalette resolvedPalette = palette.resolve(QQuickTheme::palette(QQuickTheme::System));
     d->setPalette_helper(resolvedPalette);
 }
 
