@@ -40,6 +40,7 @@
 #include "qquickmenubaritem_p.h"
 #include "qquickmenubar_p.h"
 #include "qquickpopupitem_p_p.h"
+#include "qquickpopuppositioner_p_p.h"
 #include "qquickaction_p.h"
 
 #include <QtGui/qevent.h>
@@ -172,6 +173,19 @@ static const int SUBMENU_DELAY = 225;
     \sa {Customizing Menu}, MenuItem, {Menu Controls}, {Popup Controls}
 */
 
+/*!
+    \qmlproperty bool QtQuick.Controls::Menu::focus
+
+    This property holds whether the popup wants focus.
+
+    When the popup actually receives focus, \l activeFocus will be \c true.
+    For more information, see \l {Keyboard Focus in Qt Quick}.
+
+    The default value is \c false.
+
+    \sa activeFocus
+*/
+
 static const QQuickPopup::ClosePolicy cascadingSubMenuClosePolicy = QQuickPopup::CloseOnEscape | QQuickPopup::CloseOnPressOutsideParent;
 
 static bool shouldCascade()
@@ -183,6 +197,14 @@ static bool shouldCascade()
 #endif
 }
 
+class QQuickMenuPositioner : public QQuickPopupPositioner
+{
+public:
+    QQuickMenuPositioner(QQuickMenu *menu) : QQuickPopupPositioner(menu) { }
+
+    void reposition() override;
+};
+
 QQuickMenuPrivate::QQuickMenuPrivate()
 {
     Q_Q(QQuickMenu);
@@ -204,6 +226,7 @@ void QQuickMenuPrivate::insertItem(int index, QQuickItem *item)
     if (complete)
         resizeItem(item);
     QQuickItemPrivate::get(item)->addItemChangeListener(this, QQuickItemPrivate::Destroyed | QQuickItemPrivate::Parent);
+    QQuickItemPrivate::get(item)->updateOrAddGeometryChangeListener(this, QQuickGeometryChange::Width);
     contentModel->insert(index, item);
 
     QQuickMenuItem *menuItem = qobject_cast<QQuickMenuItem *>(item);
@@ -228,6 +251,7 @@ void QQuickMenuPrivate::removeItem(int index, QQuickItem *item)
     contentData.removeOne(item);
 
     QQuickItemPrivate::get(item)->removeItemChangeListener(this, QQuickItemPrivate::Destroyed | QQuickItemPrivate::Parent);
+    QQuickItemPrivate::get(item)->removeItemChangeListener(this, QQuickItemPrivate::Geometry);
     item->setParentItem(nullptr);
     contentModel->remove(index);
 
@@ -242,41 +266,6 @@ void QQuickMenuPrivate::removeItem(int index, QQuickItem *item)
     }
 }
 
-void QQuickMenuPrivate::createAndAppendItem(QObject *object)
-{
-    Q_Q(QQuickMenu);
-    QQuickItem *item = qobject_cast<QQuickItem *>(object);
-    if (!item) {
-        if (QQuickAction *action = qobject_cast<QQuickAction *>(object))
-            item = createItem(action);
-        else if (QQuickMenu *menu = qobject_cast<QQuickMenu *>(object))
-            item = createItem(menu);
-    }
-
-    if (item) {
-        if (QQuickItemPrivate::get(item)->isTransparentForPositioner()) {
-            QQuickItemPrivate::get(item)->addItemChangeListener(this, QQuickItemPrivate::SiblingOrder);
-            item->setParentItem(contentItem);
-        } else if (contentModel->indexOf(item, nullptr) == -1) {
-            q->addItem(item);
-        }
-    } else {
-        contentData.append(object);
-    }
-}
-
-void QQuickMenuPrivate::recreateItems()
-{
-    // removeItem() will remove stuff from contentData, so we have to make a copy of it.
-    const auto originalContentData = contentData;
-
-    while (contentModel->count() > 0)
-        removeItem(0, itemAt(0));
-
-    for (QObject *object : originalContentData)
-        createAndAppendItem(object);
-}
-
 QQuickItem *QQuickMenuPrivate::beginCreateItem()
 {
     Q_Q(QQuickMenu);
@@ -384,27 +373,46 @@ void QQuickMenuPrivate::itemDestroyed(QQuickItem *item)
         removeItem(index, item);
 }
 
-void QQuickMenuPrivate::itemGeometryChanged(QQuickItem *, QQuickGeometryChange, const QRectF &)
+void QQuickMenuPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange, const QRectF &)
 {
-    if (complete)
+    if (!complete)
+        return;
+
+    if (item == contentItem) {
+        // The contentItem's geometry changed, so resize any items
+        // that don't have explicit widths set so that they fill the width of the menu.
         resizeItems();
+    } else {
+        // The geometry of an item in the menu changed. If the item
+        // doesn't have an explicit width set, make it fill the width of the menu.
+        resizeItem(item);
+    }
 }
 
-void QQuickMenuPrivate::reposition()
+QQuickPopupPositioner *QQuickMenuPrivate::getPositioner()
 {
     Q_Q(QQuickMenu);
-    if (parentMenu) {
-        if (cascade) {
-            if (popupItem->isMirrored())
-                q->setPosition(QPointF(-q->width() - parentMenu->leftPadding() + q->overlap(), -q->topPadding()));
-            else if (parentItem)
-                q->setPosition(QPointF(parentItem->width() + parentMenu->rightPadding() - q->overlap(), -q->topPadding()));
+    if (!positioner)
+        positioner = new QQuickMenuPositioner(q);
+    return positioner;
+}
+
+void QQuickMenuPositioner::reposition()
+{
+    QQuickMenu *menu = static_cast<QQuickMenu *>(popup());
+    QQuickMenuPrivate *p = QQuickMenuPrivate::get(menu);
+    if (p->parentMenu) {
+        if (p->cascade) {
+            if (p->popupItem->isMirrored())
+                menu->setPosition(QPointF(-menu->width() - p->parentMenu->leftPadding() + menu->overlap(), -menu->topPadding()));
+            else if (p->parentItem)
+                menu->setPosition(QPointF(p->parentItem->width() + p->parentMenu->rightPadding() - menu->overlap(), -menu->topPadding()));
         } else {
-            q->setPosition(QPointF(parentMenu->x() + (parentMenu->width() - q->width()) / 2,
-                                   parentMenu->y() + (parentMenu->height() - q->height()) / 2));
+            menu->setPosition(QPointF(p->parentMenu->x() + (p->parentMenu->width() - menu->width()) / 2,
+                                      p->parentMenu->y() + (p->parentMenu->height() - menu->height()) / 2));
         }
     }
-    QQuickPopupPrivate::reposition();
+    QQuickPopupPositioner::reposition();
 }
 
 bool QQuickMenuPrivate::prepareEnterTransition()
@@ -485,10 +493,12 @@ void QQuickMenuPrivate::onItemTriggered()
     if (!item)
         return;
 
-    if (QQuickMenu *subMenu = item->subMenu())
-        subMenu->popup(subMenu->itemAt(0));
-    else
+    if (QQuickMenu *subMenu = item->subMenu()) {
+        auto subMenuPrivate = QQuickMenuPrivate::get(subMenu);
+        subMenu->popup(subMenuPrivate->firstEnabledMenuItem());
+    } else {
         q->dismiss();
+    }
 }
 
 void QQuickMenuPrivate::onItemActiveFocusChanged()
@@ -617,7 +627,7 @@ bool QQuickMenuPrivate::activateNextItem()
     int count = contentModel->count();
     while (++index < count) {
         QQuickItem *item = itemAt(index);
-        if (!item || !item->activeFocusOnTab())
+        if (!item || !item->activeFocusOnTab() || !item->isEnabled())
             continue;
         setCurrentIndex(index, Qt::TabFocusReason);
         return true;
@@ -630,7 +640,7 @@ bool QQuickMenuPrivate::activatePreviousItem()
     int index = currentIndex;
     while (--index >= 0) {
         QQuickItem *item = itemAt(index);
-        if (!item || !item->activeFocusOnTab())
+        if (!item || !item->activeFocusOnTab() || !item->isEnabled())
             continue;
         setCurrentIndex(index, Qt::BacktabFocusReason);
         return true;
@@ -638,19 +648,45 @@ bool QQuickMenuPrivate::activatePreviousItem()
     return false;
 }
 
+QQuickMenuItem *QQuickMenuPrivate::firstEnabledMenuItem() const
+{
+    for (int i = 0; i < contentModel->count(); ++i) {
+        QQuickItem *item = itemAt(i);
+        if (!item || !item->isEnabled())
+            continue;
+
+        QQuickMenuItem *menuItem = qobject_cast<QQuickMenuItem *>(item);
+        if (!menuItem)
+            continue;
+
+        return menuItem;
+    }
+    return nullptr;
+}
+
 void QQuickMenuPrivate::contentData_append(QQmlListProperty<QObject> *prop, QObject *obj)
 {
     QQuickMenu *q = qobject_cast<QQuickMenu *>(prop->object);
     QQuickMenuPrivate *p = QQuickMenuPrivate::get(q);
 
-    if (!p->complete) {
-        // Don't add items until we're complete, as the delegate could change in the meantime.
-        // We'll add it to contentData and create it when we're complete.
-        p->contentData.append(obj);
-        return;
+    QQuickItem *item = qobject_cast<QQuickItem *>(obj);
+    if (!item) {
+        if (QQuickAction *action = qobject_cast<QQuickAction *>(obj))
+            item = p->createItem(action);
+        else if (QQuickMenu *menu = qobject_cast<QQuickMenu *>(obj))
+            item = p->createItem(menu);
     }
 
-    p->createAndAppendItem(obj);
+    if (item) {
+        if (QQuickItemPrivate::get(item)->isTransparentForPositioner()) {
+            QQuickItemPrivate::get(item)->addItemChangeListener(p, QQuickItemPrivate::SiblingOrder);
+            item->setParentItem(p->contentItem);
+        } else if (p->contentModel->indexOf(item, nullptr) == -1) {
+            q->addItem(item);
+        }
+    } else {
+        p->contentData.append(obj);
+    }
 }
 
 int QQuickMenuPrivate::contentData_count(QQmlListProperty<QObject> *prop)
@@ -1363,7 +1399,7 @@ void QQuickMenu::componentComplete()
 {
     Q_D(QQuickMenu);
     QQuickPopup::componentComplete();
-    d->recreateItems();
+    d->resizeItems();
 }
 
 void QQuickMenu::contentItemChange(QQuickItem *newItem, QQuickItem *oldItem)
@@ -1371,10 +1407,14 @@ void QQuickMenu::contentItemChange(QQuickItem *newItem, QQuickItem *oldItem)
     Q_D(QQuickMenu);
     QQuickPopup::contentItemChange(newItem, oldItem);
 
-    if (oldItem)
+    if (oldItem) {
         QQuickItemPrivate::get(oldItem)->removeItemChangeListener(d, QQuickItemPrivate::Children);
-    if (newItem)
+        QQuickItemPrivate::get(oldItem)->removeItemChangeListener(d, QQuickItemPrivate::Geometry);
+    }
+    if (newItem) {
         QQuickItemPrivate::get(newItem)->addItemChangeListener(d, QQuickItemPrivate::Children);
+        QQuickItemPrivate::get(newItem)->updateOrAddGeometryChangeListener(d, QQuickGeometryChange::Width);
+    }
 
     d->contentItem = newItem;
 }
@@ -1426,7 +1466,8 @@ void QQuickMenu::keyPressEvent(QKeyEvent *event)
             }
         } else {
             if (QQuickMenu *subMenu = d->currentSubMenu()) {
-                subMenu->popup(subMenu->itemAt(0));
+                auto subMenuPrivate = QQuickMenuPrivate::get(subMenu);
+                subMenu->popup(subMenuPrivate->firstEnabledMenuItem());
                 event->accept();
             }
         }
@@ -1451,12 +1492,12 @@ void QQuickMenu::timerEvent(QTimerEvent *event)
 
 QFont QQuickMenu::defaultFont() const
 {
-    return QQuickControlPrivate::themeFont(QPlatformTheme::MenuFont);
+    return QQuickTheme::font(QQuickTheme::Menu);
 }
 
 QPalette QQuickMenu::defaultPalette() const
 {
-    return QQuickControlPrivate::themePalette(QPlatformTheme::MenuPalette);
+    return QQuickTheme::palette(QQuickTheme::Menu);
 }
 
 #if QT_CONFIG(accessibility)
