@@ -370,7 +370,7 @@ QQuickOverlay *QQuickOverlay::overlay(QQuickWindow *window)
         QQuickItem *content = window->contentItem();
         // Do not re-create the overlay if the window is being destroyed
         // and thus, its content item no longer has a window associated.
-        if (content->window()) {
+        if (content && content->window()) {
             overlay = new QQuickOverlay(window->contentItem());
             window->setProperty(name, QVariant::fromValue(overlay));
         }
@@ -643,6 +643,11 @@ QQuickOverlay *QQuickOverlayAttached::overlay() const
 
     The property can be attached to any popup.
 
+    For example, to change the color of the background dimming for a modal
+    popup, the following code can be used:
+
+    \snippet qtquickcontrols2-overlay-modal.qml 1
+
     \sa Popup::modal
 */
 QQmlComponent *QQuickOverlayAttached::modal() const
@@ -671,6 +676,11 @@ void QQuickOverlayAttached::setModal(QQmlComponent *modal)
 
     The property can be attached to any popup.
 
+    For example, to change the color of the background dimming for a modeless
+    popup, the following code can be used:
+
+    \snippet qtquickcontrols2-overlay-modeless.qml 1
+
     \sa Popup::dim
 */
 QQmlComponent *QQuickOverlayAttached::modeless() const
