@@ -1201,12 +1201,19 @@ void QQuickSwipeDelegate::touchEvent(QTouchEvent *event)
     event->ignore();
 }
 
+void QQuickSwipeDelegate::componentComplete()
+{
+    Q_D(QQuickSwipeDelegate);
+    QQuickItemDelegate::componentComplete();
+    QQuickSwipePrivate::get(&d->swipe)->reposition(DontAnimatePosition);
+}
+
 void QQuickSwipeDelegate::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     Q_D(QQuickSwipeDelegate);
     QQuickControl::geometryChanged(newGeometry, oldGeometry);
 
-    if (!qFuzzyCompare(newGeometry.width(), oldGeometry.width())) {
+    if (isComponentComplete() && !qFuzzyCompare(newGeometry.width(), oldGeometry.width())) {
         QQuickSwipePrivate *swipePrivate = QQuickSwipePrivate::get(&d->swipe);
         swipePrivate->reposition(DontAnimatePosition);
     }
@@ -1214,7 +1221,12 @@ void QQuickSwipeDelegate::geometryChanged(const QRectF &newGeometry, const QRect
 
 QFont QQuickSwipeDelegate::defaultFont() const
 {
-    return QQuickControlPrivate::themeFont(QPlatformTheme::ListViewFont);
+    return QQuickTheme::font(QQuickTheme::ListView);
+}
+
+QPalette QQuickSwipeDelegate::defaultPalette() const
+{
+    return QQuickTheme::palette(QQuickTheme::ListView);
 }
 
 #if QT_CONFIG(accessibility)
